import wx
import types
GrowthGroupParams={'ParentInternName':None,'AirDamping':None,'BendResistance':None,'Clumpiness':None,'CollisionsOn':None,'Density':None,'Gravity':None,'KinkDelay':None,'KinkScale':None,'KinkStrength':None,'LengthMax':None,'LengthMin':None,'NumbPopHairs':None,'NumbVertsPerHair':None,'PositionForce':None,'PullBack':None,'PullDown':None,'PullLeft':None,'RootStiffness':None,'RootStiffnessFalloff':None,'RootWidth':None,'SpringDamping':None,'SpringStrength':None,'TipWidth':None}

scene=poser.Scene()

selectedFigure=None
slctFigName=' '
productName=' '
furProp=None

# gets the geometry belonging to hairActor and counts all polys
# and counts the polys that are in the group grpName
# returns True if all actor polys are in the group False if not
def checkHairCoverage(hairActor,grpName):
    actName=hairActor.Name()
    actGeom=hairActor.Geometry()
    polyCounts=[0,0]

    for poly in actGeom.Polygons():
        polyCounts[0]=polyCounts[0]+1
        if poly.InGroup(grpName):
            polyCounts[1]=polyCounts[1]+1

    if polyCounts[1]==polyCounts[0]:
        return True
    else:
        return False

# gets the number of hair groups in figActor and makes a list of the group names
# CALLS checkHairCoverage and makes a coresponding list of fully populated or not
# returns a two dimensional list of group names and populated bool
def getHairGroups(figActor):
    figName=figActor.Name()
    hairGrpNames=[]
    groupPop=[]
    groupCount=figActor.NumHairGroups()
    i=0

    while i<groupCount:
        hairGrp=figActor.HairGroup(i)
        hgName=hairGrp.Name()
        hairGrpNames.append(hgName)
        groupPop.append(checkHairCoverage(figActor,hgName))
        i=i+1

    actHairStats=[hairGrpNames,groupPop]
    return actHairStats


# makes a list of the polygon indices for hair growth groups that do not
# completely cover the hair actor (body part)
# returns the list
def getHairGroupIndices(hairAct,hgName):
    indices=[]
    i=0
    actGeom=hairAct.Geometry()

    for poly in actGeom.Polygons():
        if poly.InGroup(hgName):
            indices.append(i)

        i=i+1

    return indices


# CALLS getHairGroups on figActor to get the hair stats 2D list
# CALLS getHairGroupIndicies to get the polygon indices list for any
# hair growth group that is not fully populating figActor (body part)
# ... builds a growth group polygon set dictionary with key == the hairProp's internal name
# builds a new hair growth group dictionary from the GrowthGroupParams prototype
# in global space and saves it to the data prop with key == the hairProp's internal name
def buildHairGroupData(figActor):
    faIntName=figActor.InternalName()
    sufInd=faIntName.find(':')
    faIntName=faIntName[0:sufInd]
    hairStats = getHairGroups(figActor)
    groupNames=hairStats[0]
    groupPop=hairStats[1]
    indcList=[]
    actorDataStr=[[],[]]
    polyDataStr=[[],[]]
    i=0
    numGroups=len(groupNames)

    for name in groupNames:
        actorDataStr[0].append(name)
        polyDataStr[0].append(name)
        if not groupPop[i]:
            indcList=getHairGroupIndices(figActor,name)

        hggDict = dict.copy(GrowthGroupParams)
        hgGroup=figActor.HairGroup(i)
        hggDict['ParentInternName']=faIntName
        hggDict['AirDamping']=round(hgGroup.AirDamping(),4)
        hggDict['BendResistance']=round(hgGroup.BendResistance(),4)
        hggDict['Clumpiness']=round(hgGroup.Clumpiness(),4)
        hggDict['CollisionsOn']=hgGroup.CollisionsOn()
        hggDict['Density']=round(hgGroup.Density(),4)
        hggDict['Gravity']=round(hgGroup.Gravity(),4)
        hggDict['KinkDelay']=round(hgGroup.KinkDelay(),4)
        hggDict['KinkScale']=round(hgGroup.KinkScale(),4)
        hggDict['KinkStrength']=round(hgGroup.KinkStrength(),4)
        hggDict['LengthMax']=round(hgGroup.LengthMax(),4)
        hggDict['LengthMin']=round(hgGroup.LengthMin(),4)
        hggDict['NumbPopHairs']=int(hgGroup.NumbPopHairs())
        hggDict['NumbVertsPerHair']=int(hgGroup.NumbVertsPerHair())
        hggDict['PositionForce']=round(hgGroup.PositionForce(),4)
        hggDict['PullBack']=round(hgGroup.PullBack(),4)
        hggDict['PullDown']=round(hgGroup.PullDown(),4)
        hggDict['PullLeft']=round(hgGroup.PullLeft(),4)
        hggDict['RootStiffness']=round(hgGroup.RootStiffness(),4)
        hggDict['RootStiffnessFalloff']=round(hgGroup.RootStiffnessFalloff(),4)
        hggDict['RootWidth']=round(hgGroup.RootWidth(),4)
        hggDict['SpringDamping']=round(hgGroup.SpringDamping(),4)
        hggDict['SpringStrength']=round(hgGroup.SpringStrength(),4)
        hggDict['TipWidth']=round(hgGroup.TipWidth(),4)
        actorDataStr[1].append(hggDict)
        polyDataStr[1].append(indcList)
        i=i+1

    hgDict=dict(zip(actorDataStr[0],actorDataStr[1]))
    furProp.SetCustomData(faIntName+':HairGroups',hgDict,0,0)
    pgDict=dict(zip(polyDataStr[0],polyDataStr[1]))
    furProp.SetCustomData(faIntName+':PolyGroups',pgDict,0,0)

    return None

# iterates the list of figure actors (body parts) returned by the poser
# figure.Actors() Method and tests each one for hair groups
# CALLS buildHairGroupData on actors that have hair group(s)
def iterateFigureActors(figureName):
    furFigure=scene.Figure(figureName)
    furActList = furFigure.Actors()
    for furAct in furActList:
        hairGroupCount=furAct.NumHairGroups()
        if hairGroupCount > 0:
            buildHairGroupData(furAct)

    return None


# gets a shader node's input list and builds a parameters dictionary in an input list dictionary and returns the comp dictionary
def getInputNodeData_byInputs(shaderNode):
    inputList=shaderNode.Inputs()
    if shaderNode.Type()=='poser':
        inputList=inputList[0:20]

    inNames=[]
    for ndIn in inputList:
        inNames.append(ndIn.InternalName())
    snParmDict=dict.fromkeys(inNames,None)
    for np in inputList:
        inParmList=['Value','InType','InNode','NdType']
        inParmDict=dict.fromkeys(inParmList,None)
        parmVal=np.Value()
        inParmDict['Value']=parmVal
        if type(parmVal) is types.TupleType:
            inParmDict['InType']='Color'
        else:
            inParmDict['InType']='Float'
            parmStr=str(parmVal)
            if parmStr.find('Runtime')>=0:
                parmStr=parmStr.replace('\\',':')
                inda=parmStr.index(':Runtime')
                indb=len(parmStr)
                parmStr=parmStr[inda:indb]
                inParmDict['Value']=parmStr
                inParmDict['InType']='String'

        plugIn=np.InNode()
        if plugIn:
            inputName=plugIn.InternalName()
            nodeType=plugIn.Type()
            inParmDict['InNode']=inputName
            inParmDict['NdType']=nodeType

        snParmDict[np.InternalName()]=inParmDict
    return snParmDict


# iterates the scene actors and builds a list of HairProp Actors
# returns the hairPropActs list
def getHairProps():
    hairPropActs=[]
    allSceneActs=[]
    allSceneActs=scene.Actors()
    for actor in allSceneActs:
        if actor.IsHairProp():
            hairPropActs.append(actor)

    return hairPropActs

# gets the shader tree for the 'Hair materials and iterates the nodes CALLing 'getInputNodeData_byInputs'
# and saves the input parameter dictionary with a key based on the actor name & node name to the data prop
def iterateHairPropActors(hairPropActList):
    for hpAct in hairPropActList:
        hairMat=hpAct.Material('Hair')
        actorName=hpAct.Name()
        shaderTree=hairMat.ShaderTree()
        nodeList=[[],[]]
        for node in shaderTree.Nodes():
            psParmDict=getInputNodeData_byInputs(node)
            loc=node.Location()
            l=psParmDict.setdefault('Location',loc)
            nodeType=node.Type()
            nodeList[0].append(nodeType)
            nodeName=node.InternalName()
            nodeList[1].append(nodeName)
            keyStr=actorName+':'+nodeName
            furProp.SetCustomData(keyStr,psParmDict,0,0)
        ndLst=actorName+':TreeNodes'
        furProp.SetCustomData(ndLst,nodeList,0,0)

    return None

# START OF MAIN SCRIPT LOOP_____________________________________________>>
#_______________________________________________________________________>>
userInputDlg=wx.TextEntryDialog(parent=None,message='Please Enter Fur Product Name')
userInputDlg.ShowModal()
productName=userInputDlg.GetValue()
userInputDlg=None
selectedFigure=scene.CurrentFigure()
scnActors=scene.Actors()
for act in scnActors:
    if act.InternalName()=='FurCreatorProp':
        furProp=act
        break

if not selectedFigure:
    selectMsg=wx.MessageDialog(parent=None,message='Please Select Figure',style=wx.CENTER)
    selectMsg.ShowModal()
else:
    slctFigName=selectedFigure.Name()
    if not furProp:
        loadPropMsg=wx.MessageDialog(parent=None,message='Please Load the Fur Creator Prop',style=wx.CENTER)
        loadPropMsg.ShowModal()
    else:
        furProp.SetName(productName)
        iterateFigureActors(slctFigName)

hairPropList=getHairProps()
iterateHairPropActors(hairPropList)
figureName=selectedFigure.Name()
furProp.SetCustomData('Target_Figure',figureName,0,0)
msgDlg=wx.MessageDialog(parent=None,message=figureName+' Fur written to '+furProp.Name(),style=wx.CENTER)
msgDlg.ShowModal()


