# import wx
scene=poser.Scene()

def setupInputNode(dctName,siNode,shdTree):
    newNodes=[]
    siDict=furProp.CustomData(dctName)
    ndLoc=siDict['Location']
    siNode.SetLocation(ndLoc[0],ndLoc[1])
    siInputs=siNode.Inputs()
    for siInput in siInputs:
        siName=siInput.InternalName()
        siInpDict=siDict[siName]
        siValue=siInpDict['Value']
        siType=siInpDict['InType']
        if siType=='Float':
            siInput.SetFloat(siValue)
        elif siType=='Color':
            siInput.SetColor(siValue[0],siValue[1],siValue[2])
        elif siType=='String':
            siInput.SetString(siValue)

        piNodeName=siInpDict['InNode']
        piNodeType=siInpDict['NdType']
        if piNodeName:
            piNode=shdTree.NodeByInternalName(piNodeName)
            if not piNode:
                piNode=shdTree.CreateNode(piNodeType)
                if piNode:
                    piNode.SetName(piNodeName)
                    newNodes.append(piNode)

            shdTree.AttachTreeNodes(siNode,siName,piNode)
    return newNodes

def setupShaderTree(rnName,pName,shdrTree):
    rNode=shdrTree.NodeByInternalName(rnName)
    rNodeInputs=rNode.Inputs()
    rNodeInputs=rNodeInputs[0:20]
    fpRnodeInDict=furProp.CustomData(pName+':'+rnName)
    rnLoc=fpRnodeInDict['Location']
    rNode.SetLocation(rnLoc[0],rnLoc[1])
    xtPlugList=[]
    for rnInput in rNodeInputs:
        inName=rnInput.InternalName()
        inDatDict=fpRnodeInDict[inName]
        inType=rnInput.Type()
        if inType==0:
            rnInput.SetFloat(inDatDict['Value'])
        elif inType==1:
            inColor=inDatDict['Value']
            rnInput.SetColor(inColor[0],inColor[1],inColor[2])

        rnPlugIn=inDatDict['InNode']
        piType=inDatDict['NdType']
        nuPlugList=None
        if rnPlugIn:
            nuNode=shdrTree.CreateNode(piType)
            if nuNode:
                nuNode.SetName(rnPlugIn)
                shdrTree.AttachTreeNodes(rNode,inName,nuNode)
                nuPlugList=setupInputNode(pName+':'+rnPlugIn,nuNode,shdrTree)
                xtPlugList.append(nuPlugList)

    for xtPlugs in xtPlugList:
        for xtPlugIn in xtPlugs:
            xtPiName=xtPlugIn.InternalName()
            xxtPlugIns=setupInputNode(pName+':'+xtPiName,xtPlugIn,shdrTree)
            for xxtPlugIn in xxtPlugIns:
                xxtPiName=xxtPlugIn.InternalName()
                setupInputNode(pName+':'+xxtPiName,xxtPlugIn,shdrTree)
            
    return None

def configMatsForHairProp(fActHP):
    hpName=fActHP.Name()
    hpMat=fActHP.Material('Hair')
    matShader=hpMat.ShaderTree()
    initNodeList=matShader.Nodes()
    psNodeName=''
    i=0
    for node in initNodeList:
        nType=node.Type()
        if nType=='poser':
            psNodeName=node.InternalName()
        else:
            matShader.DeleteNode(i)
        i=i+1

    nType=None
    if len(psNodeName)>0:
        setupShaderTree(psNodeName,hpName,matShader)
        
    return None

def addPolysToHairGroup(frAct,fpKy,fpPList):
    frActGeom=frAct.Geometry()
    if len(fpPList)>0:
        for plyIndx in fpPList:
            frPoly=frActGeom.Polygon(plyIndx)
            frPoly.IncludeInGroup(fpKy)
    else:
        n=frActGeom.NumPolygons()
        i=0
        while i < n:
            frPoly=frActGeom.Polygon(i)
            frPoly.IncludeInGroup(fpKy)
            i=i+1
    
    return None


def buildActHairGroup(fAct,fpKey,fpHD,fpPL):
    actHG=furAct.CreateHairGroup(fpKey)
    addPolysToHairGroup(furAct,fpKey,fpPL)
    actHG.SetLengthMin(fpHD['LengthMin'])
    actHG.SetLengthMax(fpHD['LengthMax'])
    actHG.SetPullBack(fpHD['PullBack'])
    actHG.SetPullDown(fpHD['PullDown'])
    actHG.SetPullLeft(fpHD['PullLeft'])
    actHG.GrowHair()
    actHG.SetShowPopulated(0)
    actHG.SetDensity(fpHD['Density'])
    actHG.SetTipWidth(fpHD['TipWidth'])
    actHG.SetRootWidth(fpHD['RootWidth'])
    actHG.SetClumpiness(fpHD['Clumpiness'])
    actHG.SetKinkStrength(fpHD['KinkStrength'])
    actHG.SetKinkScale(fpHD['KinkScale'])
    actHG.SetKinkDelay(fpHD['KinkDelay'])
    actHG.SetNumbVertsPerHair(fpHD['NumbVertsPerHair'])
    actHG.SetGravity(fpHD['Gravity'])
    actHG.SetSpringStrength(fpHD['SpringStrength'])
    actHG.SetBendResistance(fpHD['BendResistance'])
    actHG.SetRootStiffness(fpHD['RootStiffness'])
    actHG.SetRootStiffnessFalloff(fpHD['RootStiffnessFalloff'])
    actHG.SetSpringDamping(fpHD['SpringDamping'])
    actHG.SetAirDamping(fpHD['AirDamping'])
    actHG.SetCollisionsOn(fpHD['CollisionsOn'])
    actHG.SetNumbPopHairs(fpHD['NumbPopHairs'])
    actHP= actHG.HairProp()
    return actHP

def createActHairGroups(fAct,fpHgDict,fpPyDict):
    fpKeyList=fpHgDict.keys()
    if len(fpKeyList)>0:
        for fpKey in fpKeyList:
            fpHG=fpHgDict[fpKey]
            fpPL=fpPyDict[fpKey]
            fActHP = buildActHairGroup(fAct,fpKey,fpHG,fpPL)
            configMatsForHairProp(fActHP)
         
    return None

furProp=scene.ActorByInternalName('FurCreatorProp')
furFigName=furProp.CustomData('Target_Figure')
furFig=scene.Figure(furFigName)
furActList=furFig.Actors()
for furAct in furActList:
    faIntName=furAct.InternalName()
    sfxInd=faIntName.find(':')
    fpActName=faIntName[0:sfxInd]
    hgsKey=str(fpActName)+':HairGroups'
    plysKey=str(fpActName)+':PolyGroups'
    fpHairGroupDct=furProp.CustomData(hgsKey)
    fpPolyGroupDct=furProp.CustomData(plysKey)
    if fpHairGroupDct:
        createActHairGroups(furAct,fpHairGroupDct,fpPolyGroupDct)
    
scene.SelectActor(furProp)
scene.DeleteCurrentProp()
